package compute;

import java.util.concurrent.CountDownLatch;

import data.Matrix;

public abstract class Process extends Thread {
    private final CountDownLatch barrier;
    protected final Matrix target;
    protected final Function rule;
	private long startTime;
	
	public Process(Matrix target, Function rule, CountDownLatch latch)  {
		this.barrier = latch; 
		this.target = target;  
		this.rule = rule;
	}
	
	public abstract void body( );
	
	public void run( )  {
		startTime = System.currentTimeMillis( );
        body( );
		barrier.countDown( );
	}
	
	public void complete( )  {
		try  {
			barrier.await( );
		}  catch(InterruptedException e)  {  }
	}
	
	public long runtime( )  {
		return System.currentTimeMillis( ) - startTime;
	}
    
}
