package data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.Random;

import javax.annotation.processing.Processor;

import compute.Function;

public final class Matrix {
   private int x, y; // non-final fields for re-sizing
   private final ArrayList<ArrayList<Double>> elements;	
   
	public Matrix(int x, int y)  {
		this.x = x;  this.y = y;
		elements = new ArrayList<ArrayList<Double>>(y);
		for (int i = 0;  i < y;  ++i)  {
			ArrayList<Double> row = new ArrayList<Double>(x);
			elements.add(row);
			for (int j = 0;  j < x;  ++j)  
				row.add(j, 0.0);
		}
	} 
	
	public void random( )  {
		Random generator = new Random( );
		generator.setSeed(new GregorianCalendar( ).getTimeInMillis( ));
		for (ArrayList<Double> row : elements)  
			for (int i = 0;  i < x;  ++i)
				row.set(i, generator.nextDouble( ));
	}
	
	public double trace( )  {
		double sum = 0.0;
		for (int i = 0;  i < Math.min(x, y);  ++i) 
			sum += get(i, i);
		return sum;	
	}

	@Override
	public int hashCode( )  {
		return (int)trace( );
	}
	
	@Override
	public boolean equals(Object o)  {
		if (!(o instanceof Matrix))
			return false;
		Matrix mirror = (Matrix)o;
		if (mirror.x != x || mirror.y != y)
			return false;
		for (int i = 0;  i < y;  ++i)
			for (int j = 0;  j < x;  ++j) 
				if (get(j, i) != mirror.get(j, i))
					return false;
		return true;
	}	
	
    @Override
    public Matrix clone( ) {
    	Matrix companion = new Matrix(x, y);
		for (int row = 0;  row < y;  ++row)
			Collections.copy(companion.elements.get(row), elements.get(row));
    	return companion;
    }
    
    
      
	public void print( )  {
		for (ArrayList<Double> row : elements)  {  
			for (int i = 0;  i < x;  ++i)  
				System.out.printf("%.2f ", row.get(i));
		    System.out.println( );
	    }
	}
	
	public double get(int x, int y)  {
		ArrayList<Double> row = elements.get(y);	
		return row.get(x);
	}
	
	public void set(int x, int y, double v)  {
		ArrayList<Double> row = elements.get(y);
		row.set(x, v);		
	}
	
    public void process(Function f)  {
		for (int i = 0;  i < y;  ++i)
			for (int j = 0;  j < x;  ++j)  {
				set(j, i, f.compute(get(j, i)));
			}
    }

    public void processUL(Function f) {
		for (int i = 0;  i < y;  ++i)
			for (int j = 0;  j < x - i;  ++j) 				
				set(j, i, f.compute(get(j, i)));
    }
    
    public void processLR(Function f) {
		for (int i = 1;  i < y;  ++i)
			for (int j = 1;  j + i <= x;  ++j) 
				set(x - j, y - i, f.compute(get(x - j, y - i)));
    }    
    
}