package experiments;

import data.Matrix;

import java.util.concurrent.CountDownLatch;

import compute.Function;
import compute.Process;

public final class Parallel {
	public final static int SIZE = 100;
   
	public long processMatrix(Matrix m, Function f)  {
		CountDownLatch finish = new CountDownLatch(1);	
		Process worker = new Process(m, f, finish)  {
			public void body( )  {
				target.process(rule);
			}			
		};
		worker.start( );
		worker.complete( );
		return worker.runtime( );
	}
	
	public long processTriangular(Matrix m, Function f)  {
		CountDownLatch team = new CountDownLatch(2);
		Process workerUL = new Process(m, f, team)  {
			public void body( )  {
				target.processUL(rule);
			}
		},
		workerLR = new Process(m, f, team)  {
			public void body( )  {
				target.processLR(rule);
			}
		};		
		workerUL.start( );
		workerLR.start( );
		workerUL.complete( );  // will wait for LR as well
		return Math.max(workerLR.runtime( ), workerUL.runtime( ));
	}
	
	public static void main(String... args)  {
		Parallel engine = new Parallel( );
		Function crunchNumbers = new Identity( );
		Matrix m = new Matrix(SIZE, SIZE);
		m.random( );
		long sequentialTime = engine.processMatrix(m, crunchNumbers),
		     parallelTime = engine.processTriangular(m, crunchNumbers);
		System.out.println("measured scalabilty " + ((double)sequentialTime / parallelTime) );
	}
}
